# Examples

## Real-world & production-ready example projects with Prisma

Here are a number of open source projects that demonstrate usage of Prisma in **real world applications**. Feel free to submit a PR to add your own production application if it is open source. 

- [Backend for a real world grading application](https://github.com/2color/real-world-grading-app) (incl video and written tutorials)
- [Tottem – Fullstack app for "social library management"](https://github.com/poulainv/tottem) (based on Next.js)

## Ready-to-run example projects

You can find a number of **ready-to-run** example projects that demonstrate usage of Prisma in various use cases in the [`prisma-examples`](https://github.com/prisma/prisma-examples/) repository.

### TypeScript

#### Fullstack

| Demo                                                                                                                | Description                                                                                                                                                                                       |
| :------------------------------------------------------------------------------------------------------------------ | :------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------ |
| [`rest-nextjs-api-routes`](https://github.com/prisma/prisma-examples/tree/latest/typescript/rest-nextjs-api-routes) | [Next.js](https://nextjs.org/) app (React) with a REST API (using [Next.js API routes](https://nextjs.org/docs/api-routes/introduction))                                                          |
| [`rest-nextjs-express`](https://github.com/prisma/prisma-examples/tree/latest/typescript/rest-nextjs-express)       | [Next.js](https://nextjs.org/) app (React) with a REST API (using [Express](https://expressjs.com/))                                                                                              |
| [`graphql-nextjs`](https://github.com/prisma/prisma-examples/tree/latest/typescript/graphql-nextjs)                 | [Next.js](https://nextjs.org/) app (React) with a GraphQL API (using [Apollo Server](https://github.com/apollographql/apollo-server) and [Nexus Schema](https://github.com/graphql-nexus/schema)) |

#### Backend only

| Demo                                                                                                              | Description                                                                                                                                                                          |
| :---------------------------------------------------------------------------------------------------------------- | :----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| [`graphql-apollo-server`](https://github.com/prisma/prisma-examples/tree/latest/typescript/graphql-apollo-server) | GraphQL server based on [`apollo-server`](https://www.apollographql.com/docs/apollo-server/) and [Nexus Schema](https://github.com/graphql-nexus/schema)                             |
| [`graphql-auth`](https://github.com/prisma/prisma-examples/tree/latest/typescript/graphql-auth)                   | GraphQL server with email-password authentication & permissions                                                                                                                      |
| [`graphql-sdl-first`](https://github.com/prisma/prisma-examples/tree/latest/typescript/graphql-sdl-first)         | GraphQL server based on the SDL-first approach of [`graphql-tools`](https://www.apollographql.com/docs/graphql-tools/)                                                               |
| [`graphql-subscriptions`](https://github.com/prisma/prisma-examples/tree/latest/typescript/subscriptions-pubsub)  | GraphQL server with realtime subscriptions based on [`apollo-server`](https://www.apollographql.com/docs/apollo-server/) and [Nexus Schema](https://github.com/graphql-nexus/schema) |
| [`graphql-typegraphql`](https://github.com/prisma/prisma-examples/tree/latest/typescript/graphql-typegraphql)     | GraphQL server based on [`graphql-yoga`](https://github.com/prisma-labs/graphql-yoga) and [TypeGraphQL](https://github.com/MichalLytek/type-graphql)                                 |
| [`graphql`](https://github.com/prisma/prisma-examples/tree/latest/typescript/graphql)                             | GraphQL server based on [`graphql-yoga`](https://github.com/prisma-labs/graphql-yoga) and [Nexus Schema](https://github.com/graphql-nexus/schema)                                    |
| [`grpc`](https://github.com/prisma/prisma-examples/tree/latest/typescript/grpc)                                   | gRPC API including runnable client scripts for testing                                                                                                                               |
| [`postgis-express`](https://github.com/prisma/prisma-examples/tree/latest/typescript/postgis-express)             | Demo of spatial queries using [Postgis](http://postgis.net/) and [Express](https://expressjs.com/)                                                                                   |
| [`rest-express`](https://github.com/prisma/prisma-examples/tree/latest/typescript/rest-express)                   | REST API with [Express](https://expressjs.com/)                                                                                                                                      |
| [`script`](https://github.com/prisma/prisma-examples/tree/latest/typescript/script)                               | Usage of Prisma Client JS in a TypeScript script                                                                                                                                     |
| [`testing-express`](https://github.com/prisma/prisma-examples/tree/latest/typescript/testing-express)             | Demo of integration tests with [Jest](https://jestjs.io/), [Supertest](https://github.com/visionmedia/supertest) and [Express](https://expressjs.com/)                               |

### JavaScript (Node.js)

#### Fullstack

| Demo                                                                                          | Description                                                                                                                              |
| :-------------------------------------------------------------------------------------------- | :--------------------------------------------------------------------------------------------------------------------------------------- |
| [`rest-nextjs`](https://github.com/prisma/prisma-examples/tree/latest/javascript/rest-nextjs) | [Next.js](https://nextjs.org/) app (React) with a REST API (using [Next.js API routes](https://nextjs.org/docs/api-routes/introduction)) |

#### Backend only

| Demo                                                                                                              | Description                                                                                                            |
| :---------------------------------------------------------------------------------------------------------------- | :--------------------------------------------------------------------------------------------------------------------- |
| [`graphql-apollo-server`](https://github.com/prisma/prisma-examples/tree/latest/javascript/graphql-apollo-server) | GraphQL server based on [`apollo-server`](https://www.apollographql.com/docs/apollo-server/)                           |
| [`graphql-auth`](https://github.com/prisma/prisma-examples/tree/latest/javascript/graphql-auth)                   | GraphQL server with email-password authentication & permissions                                                        |
| [`graphql-sdl-first`](https://github.com/prisma/prisma-examples/tree/latest/javascript/graphql-sdl-first)         | GraphQL server based on the SDL-first approach of [`graphql-tools`](https://www.apollographql.com/docs/graphql-tools/) |
| [`graphql`](https://github.com/prisma/prisma-examples/tree/latest/javascript/graphql)                             | GraphQL server based on [`graphql-yoga`](https://github.com/prisma-labs/graphql-yoga)                                  |
| [`grpc`](https://github.com/prisma/prisma-examples/tree/latest/javascript/grpc)                                   | gRPC API including runnable client scripts for testing                                                                 |
| [`rest-express`](https://github.com/prisma/prisma-examples/tree/latest/javascript/rest-express)                   | REST API with [Express](https://expressjs.com/)                                                                        |
| [`rest-fastify`](https://github.com/prisma/prisma-examples/tree/latest/javascript/rest-fastify)                   | REST API with [Fastify](https://www.fastify.io/)                                                                       |
| [`script`](https://github.com/prisma/prisma-examples/tree/latest/javascript/script)                               | Usage of Prisma Client JS in a Node.js script                                                                         |